// DesktopDemo.java

import java.awt.*;
import java.awt.event.*;

import java.io.*;

import java.net.*;

import javax.swing.*;

class DesktopDemo extends JFrame
{
   Desktop desktop;

   DesktopDemo (String title)
   {
      // Assign title to the frame window's title bar.

      super (title);

      // Tell application to automatically exit when the user selects the Close
      // menu item from the frame window's system menu.

      setDefaultCloseOperation (EXIT_ON_CLOSE);

      // Obtain the desktop.

      desktop = Desktop.getDesktop ();

      // Build top panel, consisting of a text-entry field for specifying a
      // URI, a mail recipient, or a filename.

      JPanel pnlTop = new JPanel ();

      pnlTop.add (new JLabel ("Enter URI or filename (as appropriate):"));

      final JTextField txtText = new JTextField (30);
      pnlTop.add (txtText);

      getContentPane ().add (pnlTop, BorderLayout.NORTH);

      // Build bottom panel, consisting of five mutually-exclusive radio
      // buttons for selecting an action.

      JPanel pnlBottom = new JPanel ();

      final ButtonGroup bg = new ButtonGroup ();

      JRadioButton rbBrowse = new JRadioButton ("Browse");
      rbBrowse.setMnemonic (KeyEvent.VK_B);

      ActionListener al;
      al = new ActionListener ()
           {
               public void actionPerformed (ActionEvent e)
               {
                  try
                  {
                      String uri = txtText.getText ();
                      desktop.browse (new URI (uri));
                  }
                  catch (IOException ioe)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "I/O problem");
                  }
                  catch (URISyntaxException use)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "Bad URI syntax");
                  }
               }
           };
      rbBrowse.addActionListener (al);

      if (!desktop.isSupported (Desktop.Action.BROWSE))
          rbBrowse.setEnabled (false);

      bg.add (rbBrowse);
      pnlBottom.add (rbBrowse);

      JRadioButton rbMail = new JRadioButton ("Mail");
      rbMail.setMnemonic (KeyEvent.VK_M);

      al = new ActionListener ()
           {
               public void actionPerformed (ActionEvent e)
               {
                  try
                  {
                      String mailTo = txtText.getText ();
                      if (mailTo.length () > 0)
                          desktop.mail (new URI ("mailto", mailTo, null));
                      else
                          desktop.mail ();
                  }
                  catch (IOException ioe)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "I/O problem");
                  }
                  catch (URISyntaxException use)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "Bad URI syntax");
                  }
               }
           };
      rbMail.addActionListener (al);

      if (!desktop.isSupported (Desktop.Action.MAIL))
          rbMail.setEnabled (false);

      bg.add (rbMail);
      pnlBottom.add (rbMail);

      JRadioButton rbOpen = new JRadioButton ("Open");
      rbOpen.setMnemonic (KeyEvent.VK_O);

      al = new ActionListener ()
           {
               public void actionPerformed (ActionEvent e)
               {
                  try
                  {
                      String filename = txtText.getText ();
                      desktop.open (new File (filename));
                  }
                  catch (IOException ioe)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "I/O problem");
                  }
                  catch (IllegalArgumentException iae)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "File doesn't exist");
                  }
               }
           };
      rbOpen.addActionListener (al);

      if (!desktop.isSupported (Desktop.Action.OPEN))
          rbOpen.setEnabled (false);

      bg.add (rbOpen);
      pnlBottom.add (rbOpen);

      JRadioButton rbEdit = new JRadioButton ("Edit");
      rbEdit.setMnemonic (KeyEvent.VK_E);

      al = new ActionListener ()
           {
               public void actionPerformed (ActionEvent e)
               {
                  try
                  {
                      String filename = txtText.getText ();
                      desktop.edit (new File (filename));
                  }
                  catch (IOException ioe)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "I/O problem");
                  }
                  catch (IllegalArgumentException iae)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "File doesn't exist");
                  }
               }
           };
      rbEdit.addActionListener (al);

      if (!desktop.isSupported (Desktop.Action.EDIT))
          rbEdit.setEnabled (false);

      bg.add (rbEdit);
      pnlBottom.add (rbEdit);

      JRadioButton rbPrint = new JRadioButton ("Print");
      rbPrint.setMnemonic (KeyEvent.VK_P);

      al = new ActionListener ()
           {
               public void actionPerformed (ActionEvent e)
               {
                  try
                  {
                      String filename = txtText.getText ();
                      desktop.print (new File (filename));
                  }
                  catch (IOException ioe)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "I/O problem");
                  }
                  catch (IllegalArgumentException iae)
                  {
                      JOptionPane.showMessageDialog (DesktopDemo.this,
                                                     "File doesn't exist");
                  }
               }
           };
      rbPrint.addActionListener (al);

      if (!desktop.isSupported (Desktop.Action.PRINT))
          rbPrint.setEnabled (false);

      bg.add (rbPrint);
      pnlBottom.add (rbPrint);

      getContentPane ().add (pnlBottom, BorderLayout.CENTER);

      // Resize the GUI to its preferred size.

      pack ();

      // Display GUI and start the AWT's event-dispatching thread.

      setVisible (true);
   }

   void showError (String msg)
   {
      JOptionPane.showMessageDialog (DesktopDemo.this, msg);
   }

   public static void main (String [] args)
   {
      Runnable r = new Runnable ()
                   {
                       public void run ()
                       {
                          // Determine whether or not the desktop is
                          // available. Under the Solaris and Linux operating
                          // systems, for example, the Desktop API depends on
                          // the Gnome libraries. If those libraries are not
                          // available, Desktop's isDesktopSupported() method
                          // returns false.

                          if (!Desktop.isDesktopSupported ())
                          {
                              JOptionPane.showMessageDialog (null,
                                                 "Desktop is not supported!");
                              System.exit (-1);
                          }

                          new DesktopDemo ("Desktop Demo");
                       }
                   };
      EventQueue.invokeLater (r);
   }
}
